#include <csvreader.hpp>
#include <sstream>

CSVReader :: CSVReader
 (
   string file,
   string delimiter,
   bool hasHeadLine
 )
   throw(Error)
  :
  clFile(file.c_str()),
  clDelimiter(delimiter),
  clHasHeadLine(hasHeadLine)
{
   if (!clFile.good())
   {
      throw Error("Could not open CSV File '" + file + "'");
   }

   if (clHasHeadLine)
   {
      if (!clFile.eof())
      {
         getline(clFile, clHeadLine);
      }

      clNumFields = countFields(clHeadLine);
   }

   if (!clFile.eof())
   {
      next();

      clNumFields = countFields(clNextLine);
   }
}

CSVReader :: ~CSVReader()
{
}

bool CSVReader :: hasNext()
{
   return (! clNextLine.empty());
}

void CSVReader :: next()
{
   clLine = clNextLine;

   if (! clFile.eof())
   {
      getline(clFile, clNextLine);
   }
   else
   {
      clNextLine = "";
   }
}

int CSVReader :: getNumFields()
{
   return clNumFields;
}

int CSVReader :: countFields
 (
  string &row
 )
{
   int count = 1;
   size_t pos = -1;

   while ( (pos = row.find(clDelimiter, pos+1)) != string::npos )
      ++count;

   return count;
}

string CSVReader :: getLine()
   throw(Error)
{
   return clLine;
} 

string CSVReader :: getField
 (
   int fieldNo
 )
   throw(Error)
{
   return _getField(clLine, fieldNo);
}

string CSVReader :: getHeadField
 (
   int fieldNo
 )
   throw(Error)
{
   return _getField(clHeadLine, fieldNo);
}

string CSVReader :: _getField
 (
   string &row,
   int fieldNo
 )
   throw(Error)
{
   int origFieldNo = fieldNo;

   if (fieldNo <= 0)
   {
      throw Error("field number may not be <= 0");
   }

   if (fieldNo > getNumFields())
   {
      throw Error("field number out of range");
   }

   if (fieldNo == 1)
   {
      size_t pos = row.find(clDelimiter, 0);

      if (pos == string::npos)
      {
         return row;
      }

      return row.substr(0, pos);
   }

   // test;lol;test

   size_t beginPos = -1;

   while (--fieldNo > 0)
   {
      beginPos = row.find(clDelimiter, beginPos + 1);

      if (beginPos == string::npos)
      {
         stringstream ss;
         ss << "Field number out of range! Headline has " << getNumFields() << " fields."
            << "field " << origFieldNo << " out of range" << endl << "line=" << row;

         throw Error(ss.str());
      }
   }

   size_t endPos = row.find(clDelimiter, beginPos + 1);

   if (endPos == string::npos)
   {
      return row.substr(beginPos + 1);
   }
   else
   {
      return row.substr(beginPos + 1, endPos - beginPos - 1);
   }
}


string CSVReader :: getHeadLine()
   throw(Error)
{
   if (!clHasHeadLine)
   {
      throw Error("Headline? You said, that I have no headline... :(");
   }

   return clHeadLine;
}

bool CSVReader :: hasHeadLine()
{
   return clHasHeadLine;
}
