#include <QtGui>
#include <vector>
#include <csvreader.hpp>
#include <splitter.hpp>
#include <iostream>
#include <string>

using namespace std;

class CSVGroupGui : public QWidget
{
    Q_OBJECT

public:
   CSVGroupGui();
   void showError(QString x);

private slots:
   void quit();
   void chooseInputFile();
   void chooseOutputDirectory();
   void reloadInputFile();
   void startSplitting();

private:
// Data
QString
   clInputFile;

QString
   clOutputDirectory;

CSVReader
   *clCsvReader;

// GUI
// Delimiter
QGroupBox
   *clDelimiterGroupBox;

QHBoxLayout
   *clDelimiterHBoxLayout;

QLabel
   *clDelimiterLabel;

QLineEdit
   *clDelimiterLineEdit;

// Headline
QGroupBox
   *clHeadLineGroupBox;

QVBoxLayout
   *clHeadLineVBoxLayout;

QCheckBox
   *clHeadLineCheckBox;

// Eingabe Datei
QGroupBox
   *clInputFileGroupBox;

QHBoxLayout
   *clInputFileHBoxLayout;

QLineEdit
   *clInputFileLineEdit;

QPushButton
   *clInputFileButton;

// Ausgabe Datei
QGroupBox
   *clOutputDirectoryGroupBox;

QHBoxLayout
   *clOutputDirectoryHBoxLayout;

QLineEdit
   *clOutputDirectoryLineEdit;

QPushButton
   *clOutputDirectoryButton;

// Gruppen Selektion
QGroupBox
   *clGroupSelectionGroupBox;

QVBoxLayout
   *clGroupSelectionVBoxLayout;

QComboBox
   *clGroupSelectionComboBox;

// Split
QPushButton
   *clSplitButton;

// Quit
QPushButton
   *clQuitButton;
};

CSVGroupGui :: CSVGroupGui()
{
   // INIT Data
   clCsvReader = 0;

   // Delimiter
   clDelimiterLabel = new QLabel(tr("Trennzeichen"));

   clDelimiterLineEdit = new QLineEdit(tr(";"));

   clDelimiterHBoxLayout = new QHBoxLayout;
   clDelimiterHBoxLayout->addWidget(clDelimiterLabel);
   clDelimiterHBoxLayout->addWidget(clDelimiterLineEdit);

   clDelimiterGroupBox = new QGroupBox(tr("Trennzeichen"));
   clDelimiterGroupBox->setLayout(clDelimiterHBoxLayout);

   // Headline
   clHeadLineCheckBox = new QCheckBox(tr("Kopfzeile einschalten"));
   clHeadLineCheckBox->setChecked(true);

   clHeadLineVBoxLayout = new QVBoxLayout();
   clHeadLineVBoxLayout->addWidget(clHeadLineCheckBox);

   clHeadLineGroupBox = new QGroupBox(tr("Kopfzeile"));
   clHeadLineGroupBox->setLayout(clHeadLineVBoxLayout);

   // Inputfile
   clInputFileLineEdit = new QLineEdit(tr(""));
   clInputFileLineEdit->setReadOnly(true);

   clInputFileButton = new QPushButton(tr("Datei"));

   clInputFileHBoxLayout = new QHBoxLayout();
   clInputFileHBoxLayout->addWidget(clInputFileLineEdit);
   clInputFileHBoxLayout->addWidget(clInputFileButton);

   clInputFileGroupBox = new QGroupBox("CSV Datei");
   clInputFileGroupBox->setLayout(clInputFileHBoxLayout);

   // Outputfile
   clOutputDirectoryLineEdit = new QLineEdit(tr("."));
   clOutputDirectoryLineEdit->setReadOnly(true);

   clOutputDirectoryButton = new QPushButton(tr("Ordner"));

   clOutputDirectoryHBoxLayout = new QHBoxLayout();
   clOutputDirectoryHBoxLayout->addWidget(clOutputDirectoryLineEdit);
   clOutputDirectoryHBoxLayout->addWidget(clOutputDirectoryButton);

   clOutputDirectoryGroupBox = new QGroupBox("Ausgabe Ordner");
   clOutputDirectoryGroupBox->setLayout(clOutputDirectoryHBoxLayout);

   // Gruppen Selektion
   clGroupSelectionComboBox = new QComboBox;

   clGroupSelectionVBoxLayout = new QVBoxLayout();
   clGroupSelectionVBoxLayout->addWidget(clGroupSelectionComboBox);

   clGroupSelectionGroupBox = new QGroupBox("Gruppe");
   clGroupSelectionGroupBox->setLayout(clGroupSelectionVBoxLayout);

   // Split Button
   clSplitButton = new QPushButton(tr("Gruppiere"));

   // Quit Button
   clQuitButton = new QPushButton(tr("Beenden"));

   connect(clQuitButton, SIGNAL(clicked()), this, SLOT(quit()));
   connect(clInputFileButton, SIGNAL(clicked()), this, SLOT(chooseInputFile()));
   connect(clOutputDirectoryButton, SIGNAL(clicked()), this, SLOT(chooseOutputDirectory()));
   connect(clSplitButton, SIGNAL(clicked()), this, SLOT(startSplitting()));
   connect(clHeadLineCheckBox, SIGNAL(clicked()), this, SLOT(reloadInputFile()));
   connect(clDelimiterLineEdit, SIGNAL(textChanged()), this, SLOT(reloadInputFile()));

   QVBoxLayout *layout = new QVBoxLayout;
   layout->addWidget(clDelimiterGroupBox);
   layout->addWidget(clHeadLineGroupBox);
   layout->addWidget(clInputFileGroupBox);
   layout->addWidget(clOutputDirectoryGroupBox);
   layout->addWidget(clGroupSelectionGroupBox);
   layout->addWidget(clSplitButton);
   layout->addWidget(clQuitButton);

   setLayout(layout); 

   setWindowTitle(tr("CSV Group"));
}

void CSVGroupGui :: showError(QString x)
{
    QMessageBox messageBox;
    messageBox.setWindowTitle(tr("CSV Group - Error"));
    messageBox.setText(x);
    messageBox.setStandardButtons(QMessageBox::Yes);
    messageBox.setDefaultButton(QMessageBox::Yes);
    messageBox.exec();
}

void CSVGroupGui :: quit()
{
   qApp->quit();
}

void CSVGroupGui :: reloadInputFile()
{
   cout << "LOL" << endl;

   if (clCsvReader != 0)
   {
      delete clCsvReader;
      clCsvReader = 0;
   }

   if (clInputFile != "")
   {
      try
      {
         clCsvReader = new CSVReader(
               clInputFile.toStdString(),
               clDelimiterLineEdit->text().toStdString(),
               clHeadLineCheckBox->isChecked()
         );

         while (clGroupSelectionComboBox->count())
         {
            clGroupSelectionComboBox->removeItem(0);
         }

         if (clCsvReader->hasHeadLine())
         {
            for (int i = 1; i <= clCsvReader->getNumFields(); ++i)
            {
               clGroupSelectionComboBox->addItem(
                     QString(clCsvReader->getHeadField(i).c_str())
               );

               cout << clCsvReader->getHeadField(i) << endl;
            }
         }
         else
         {
            for (int i = 1; i <= clCsvReader->getNumFields(); ++i)
            {
               QString iStr;
               iStr.setNum(i);
               clGroupSelectionComboBox->addItem(iStr);
            }
         }
      }
      catch (Error e)
      {
         cout << e.getMessage() << endl;
      }
   }
}

void CSVGroupGui :: chooseInputFile()
{
   clInputFile = QFileDialog::getOpenFileName(this,
      tr("Datei öffnen"), "./", tr("Text Dateien (*.csv *.txt)"));

   if (clInputFile == "")
   {
      clInputFileLineEdit->setText("...");
   }
   else
   {
      clInputFileLineEdit->setText(clInputFile);
   }

   reloadInputFile();
}

void CSVGroupGui :: chooseOutputDirectory()
{
   clOutputDirectory = QFileDialog::getExistingDirectory(this,
      tr("Ordner öffnen"), "./");

   if (clOutputDirectory == "")
   {
      clOutputDirectoryLineEdit->setText("...");
   }
   else
   {
      clOutputDirectoryLineEdit->setText(clOutputDirectory);
   }
}

void CSVGroupGui :: startSplitting()
{
   if (clCsvReader == 0)
   {
      showError("Es wurde noch keine CSV Datei ausgewählt");
   }

   int groupNo = clGroupSelectionComboBox->currentIndex() + 1;
   string outDir = clOutputDirectory.toStdString();

   try
   {
      Splitter s(clCsvReader);
      s.split(groupNo, outDir);
   }
   catch (Error e)
   {
      cout << e.getMessage() << endl;
   }
}

int main(int argc, char **argv)
{
    QApplication app(argc, argv);

    CSVGroupGui notepad;
    notepad.show();

    return app.exec();
}

#include "main.moc"
